@@ -16,11 +16,11 @@ import datetime
 import radvelso
 
 
-def find_schema(schema_name):
+def find_schema(schema):
   """find schema file contained in the Python package
 
    args: 
-     schema_name : name of the schema we are looking for
+     schema: the schema we are looking for
 
    output:
      full path of schema file that was found
@@ -31,7 +31,7 @@ def find_schema(schema_name):
   #root path of radvelso package (one dir up from radvelso module)
   package_dir = os.path.dirname(radvelso_dir)
   #full path of schema we are looking for
-  schema_file = f'{package_dir}/schemas/{schema_name}'
+  schema_file = f'{package_dir}/schema/{schema}'
 
   if not os.path.isfile(schema_file):
     raise ValueError(f'schema_file: {schema_file} does not exist')
@@ -39,7 +39,6 @@ def find_schema(schema_name):
   return schema_file
 
 
-
 def make_input_ppi_db(filein, 
                       stn, 
                       this_time, 
@@ -74,10 +73,14 @@ def make_input_ppi_db(filein,
   # SQLite continues without syncing as soon as it has handed data off to the operating system
   conn_ppi_db.execute("""PRAGMA synchronous=OFF;""")
 
-  schema_ppi = find_schema('schema_ppi')
-  with open(schema_ppi) as f:
+  schema = find_schema('schema')
+  with open(schema) as f:
     schema = f.read()
   conn_ppi_db.executescript(schema)
+  # improve searches with index tables
+  create_index_tables_elevation_azimuth(conn_ppi_db)
+  create_index_tables_idobs_iddata(conn_ppi_db)
+  # attach database
   conn_ppi_db.execute("ATTACH DATABASE ? AS db_all", (filein,)) 
   #select headers in desirec PPI
   order_sql = """ INSERT into header 
@@ -309,8 +312,54 @@ def create_header(conn_filememory,
                                    azimuth, range_start, range_end, 
                                    avg_elevation, time_start, time_end) )
 
+def create_index_tables_elevation_azimuth(conn):
+  """ Create index tables
+  For elevation and azimuth  
+  args:
+    conn          : connextion to sqlite file
+  output:
+    Nothing
+    index tables i sqlite file
+ 
+  """
 
-def create_info_resume_tables(filein,
+  order_sql ="""CREATE INDEX 
+                  NOMINAL_PPI_ELEVATIONS 
+                ON header (NOMINAL_PPI_ELEVATION);"""
+  conn.execute(order_sql)
+
+  order_sql ="""CREATE INDEX
+                  CENTER_AZIMUTHS 
+                ON header (CENTER_AZIMUTH);"""
+  conn.execute(order_sql)
+
+def create_index_tables_idobs_iddata(conn):
+  """ Create index tables
+  For id_obs and id_data
+  args:
+    conn          : connextion to sqlite file
+  output:
+    Nothing
+    index tables i sqlite file
+
+  """
+  
+  order_sql = """ CREATE INDEX
+                    idx1
+                  ON header(id_obs);"""
+  conn.execute(order_sql)
+  
+  order_sql = """CREATE INDEX
+                   idx2 
+                 ON data(id_obs);"""
+  conn.execute(order_sql)
+
+  order_sql = """CREATE INDEX
+                   idx3 
+                 ON data(id_data);"""
+  conn.execute(order_sql)
+
+def create_info_midas_tables(filein,
                               conn_pathfileout, 
                               n_rays,        
                               delta_range,
@@ -331,9 +380,11 @@ def create_info_resume_tables(filein,
 
   """
 
-  conn_pathfileout.execute("ATTACH DATABASE ? AS db_all", (filein,) ) 
+  conn_pathfileout.execute("""ATTACH DATABASE ? AS db_all""", (filein,) ) 
 
   # Info table
+  order_sql ="""CREATE TABLE info( NAME, DESCRIPTION, UNIT );"""
+  conn_pathfileout.execute( order_sql)
   order_sql =""" INSERT  into  info (
                    NAME, DESCRIPTION, UNIT) 
                  SELECT * 
@@ -347,11 +398,20 @@ def create_info_resume_tables(filein,
   conn_pathfileout.execute( order_sql, (delta_range, ))
 
   # Resume table 
+  order_sql ="""CREATE TABLE resume(date integer , time integer , run varchar(9));"""
+  conn_pathfileout.execute( order_sql)
   YYYYMMDD = os.path.basename(filein)[:8]
   HH = os.path.basename(filein)[8:10]
-  order_sql = "INSERT into resume values(?,?,?);"
+  order_sql = """INSERT into resume values(?,?,?);"""
   conn_pathfileout.execute( order_sql, (YYYYMMDD, HH, ops_run_name,))
+  
+  # rdb4_schema table
+  order_sql ="""CREATE TABLE rdb4_schema( schema  varchar(9) );"""
+  conn_pathfileout.execute( order_sql)
+  order_sql ="""INSERT into rdb4_schema values('radvel');"""
+  conn_pathfileout.execute( order_sql)
 
+  # commint and detach database
   conn_pathfileout.commit()
   conn_pathfileout.execute("DETACH DATABASE db_all")
 
@@ -408,11 +468,12 @@ def superobs(filein,
   # SQLite continues without syncing as soon as it has handed data off to the operating system
   conn_filememory.execute("""PRAGMA synchronous=OFF;""")
 
-  schema_memory = find_schema('schema_memory')
-  with open(schema_memory) as f:
+  schema = find_schema('schema')
+  with open(schema) as f:
     schema = f.read( )
   conn_filememory.executescript(schema)
-
+  # improve searches with index tables
+  create_index_tables_idobs_iddata(conn_filememory)
   # Construct a new Generator with the default BitGenerator (PCG64).
   rng = np.random.default_rng()
   maxsize = sys.maxsize
@@ -551,9 +612,9 @@ def combine(pathfileout, filememory):
   conn_pathfileout.execute(""" DETACH DATABASE this_avg_db """)
 
 
-def distinct_id_obs(conn_pathfileout):
-   """to check that the id_obs of the header tables have an associated id_data.
-      And there is no greater value of id_obs in the data table than in the header table.
+def collision_test(conn_pathfileout):
+   
+   """Check the collisiona and that the id_obs from the header have an associated properly to id_data.
  
    args:
     conn_pathfileout   : connection of  output averaging  sqlite file 
@@ -562,131 +623,88 @@ def distinct_id_obs(conn_pathfileout):
 
    """
    # distinct id_obs from header
+   # count id_obs from header
    order_sql = """select 
-                     count( distinct id_obs) 
-                   from header;"""
+                    count(distinct id_obs)
+                  from header;"""
    cursor   = conn_pathfileout.cursor()
    cursor.execute(order_sql)
    result   = cursor.fetchone()
    distinct_id_obs_from_header = result[0]
-   # distinct id_obs from data
    order_sql = """select 
-                     count( distinct id_obs) 
-                   from data;"""
+                    count(id_obs)
+                  from header;"""
+   cursor   = conn_pathfileout.cursor()
+   cursor.execute(order_sql)
+   result   = cursor.fetchone()
+   count_id_obs_from_header = result[0]
+   
+   # distinct id_obs from data
+   # distinct id_data from data
+   # count id_data from data
+   order_sql = """select 
+                    count(distinct id_obs)
+                  from data;"""
    cursor   = conn_pathfileout.cursor()
    cursor.execute(order_sql)
    result   = cursor.fetchone()
    distinct_id_obs_from_data = result[0]
+   order_sql = """select 
+                    count(distinct id_data)
+                  from data;"""
+   cursor   = conn_pathfileout.cursor()
+   cursor.execute(order_sql)
+   result   = cursor.fetchone()
+
+   distinct_id_data_from_data = result[0]
+   order_sql = """select 
+                    count(id_data)
+                  from data;"""
+   cursor   = conn_pathfileout.cursor()
+   cursor.execute(order_sql)
+   result   = cursor.fetchone()
+   count_id_data_from_data = result[0]
+
+
    if (distinct_id_obs_from_header !=  distinct_id_obs_from_data): 
       raise RuntimeError("""Error: id_obs is not associated properly""")
+   if (distinct_id_obs_from_header !=  count_id_obs_from_header): 
+      raise RuntimeError("""Error: id_obs is in collision""")
+   if (distinct_id_data_from_data  !=  count_id_data_from_data): 
+      raise RuntimeError("""Error: id_data is in collision""")
 
-def superobs_sqlite():
-
-  """Compute average volume scans, possibly in parallel
+def compute_average(center_time,
+         window_width,
+         desired_radar_list,
+         infile_list, 
+         pathout, 
+         outfile_struc, 
+         timing, 
+         ops_run_name, 
+         n_rays, 
+         delta_range,
+         n_cpus):
+  """launch computation of average volums scans, possibly in parallel
 
   args:
-    see argparse definitions just below for command line arguments to this function
-
+    center_time  : datetime object,  center time of the assimilation window
+    window_width : [hours] Width of the assimilation window
+    desired_radar_list : list of radars to process
+    infile_list  : list of input files to be superobbed
+    pathout      : output average sqlite file(s) 
+    outfile_struc: structure of output file name ; will be passed to strftime
+    timing       : activation of time analysis, only the first PPI in a volume scan will be averaged
+    ops_run_name : name of operational run 
+    n_rays       : number of rays
+    delta_range  : delta range of the boxes
+    n_cpus       : number of cpus for parallel execution, with n_cpus=1 joba are launched sequentially and dask is not invoked 
+ 
   output:
     Nothing
     Average sqlite file(s) are created with the '_thin' suffix
 
   """
 
-
-  parser = argparse.ArgumentParser()
-  parser.add_argument('--center_time',  default='undefined', type=str,   help="YYYYMMDDHH center time of assimilation window")
-  parser.add_argument('--window_width', default=0.,          type=float, help="[hours] width of assimilation window")
-  parser.add_argument('--radar_list', nargs="+", default='all',          help="List of radars to process")
-  parser.add_argument('--inputfiles', nargs="+", default='undefined',    help="input sqlite file to average")
-  parser.add_argument('--pathin',       default='undefined', type=str,   help="directory where are input sqlite files")
-  parser.add_argument('--infile_struc', default='*/%Y%m%d%H_ra',type=str,help="structure of input file name")
-  parser.add_argument('--pathout',      default=os.getcwd(), type=str,   help="where averaged sqlite files will be put")
-  parser.add_argument('--outfile_struc',default='%Y%m%d%H_superobbed_dvel_ground_radars.sqlite', type=str,   help="structure of output file name")
-  parser.add_argument('--timing',       action="store_true",             help="Average only one PPI for time analysis" ) 
-  parser.add_argument('--ops_run_name', default='N1',        type=str,   help="Name of operational run (will show up in rdb4_schema)" ) 
-  parser.add_argument('--n_rays',       default=20,          type=int,   help="Number of rays for averaging" )
-  parser.add_argument('--delta_range',  default=10000.,      type=float, help="Delta_range  for averaging" )
-  parser.add_argument('--n_cpus',       default=1,           type=int,   help="Number of rays for averaging" )
-
-  args = parser.parse_args()
-
-
-  #run demo case when input files unspecified
-  if args.inputfiles == 'undefined' and args.pathin == 'undefined':
-
-    print(f'superobs called with no input filename(s)')
-
-    args.center_time = '2019073100'
-    args.window_width = 6.
-    args.radar_list = ['casra', 'usdyx']
-    #option1: explicitely specify inputfiles
-    #args.inputfiles = ['/space/hall4/sitestore/eccc/cmd/a/dlo001/data/doppler_qc/doppler_qc_v0.3/sqlite_v1.0.0_qc/split_6h/USVNX/2019073100_ra',
-    #                   '/space/hall4/sitestore/eccc/cmd/a/dlo001/data/doppler_qc/doppler_qc_v0.3/sqlite_v1.0.0_qc/split_6h/CASRA/2019073100_ra']
-    #option2: specify pathin + infile_struc and let Python search for files
-    args.pathin = '/space/hall4/sitestore/eccc/cmd/a/dlo001/data/doppler_qc/doppler_qc_v0.3/sqlite_v1.0.0_qc/split_6h/'
-    args.pathout = './work'
-    args.timing = False
-    args.plot   = False
-    args.n_cpus = 40
-
-
-  print(f'superobs_sqlite starting with the following arguments:')
-  for arg in vars(args):
-    print(f'--{arg}  {getattr(args, arg)}')
-
-
-  #argument checking
-  if args.center_time == 'undefined':
-      raise ValueError('Center time of assimilation window must be provided')
-  else:
-     args.center_time = datetime.datetime.strptime(args.center_time, '%Y%m%d%H')
-
-  if np.isclose(args.window_width, 0.) :
-      raise ValueError('Window width must be provided')
-
-  if args.inputfiles != 'undefined':
-    #if inputfiles argument is provided, we use that
-    infile_list = args.inputfiles 
-
-  elif args.pathin != 'undefined': 
-    #alternatively, search for files with pathin+infile_struc 
-    if not os.path.isdir(args.pathin):
-      raise ValueError(f'pathin: {args.pathin} does not exist.')
-    search_str = args.center_time.strftime(args.infile_struc)
-    infile_list = glob.glob(f'{args.pathin}/{search_str}')
-
-  else:
-    raise ValueError('At least one of inputfiles ot pathin must be provided')
-
-  #check infile_list
-  if len(infile_list) == 0:
-    raise ValueError('infile_list is empty, we stop here')
-    sys.exit(1)
-
-  else:
-    for this_file in infile_list:
-      if not os.path.isfile(this_file):
-        raise ValueError(f'inputfiles: {this_file} does not exist.')
-
-  if not os.path.isdir(args.pathout):
-    os.mkdir(args.pathout)
-
-  center_time        = args.center_time
-  window_width       = args.window_width
-  desired_radar_list = args.radar_list
-  pathout            = args.pathout
-  outfile_struc      = args.outfile_struc
-  timing             = args.timing
-  ops_run_name       = args.ops_run_name
-  n_rays             = args.n_rays
-  delta_range        = args.delta_range
-  n_cpus             = args.n_cpus        
-
-  #we are done processing arguments
-
-
-
   #start and stop time of assimilation window
   window_t0 = center_time - datetime.timedelta(seconds=3600*window_width/2.)
   window_tf = center_time + datetime.timedelta(seconds=3600*window_width/2.)
@@ -700,8 +718,8 @@ def superobs_sqlite():
   #init large output file
   conn_pathfileout = sqlite3.connect(pathfileout)
 
-  schema_sqlmidas = find_schema('schema_sqlmidas')
-  with open(schema_sqlmidas) as f:
+  schema = find_schema('schema')
+  with open(schema) as f:
     schema = f.read()
   conn_pathfileout.executescript(schema)
 
@@ -711,22 +729,24 @@ def superobs_sqlite():
     with sqlite3.connect(this_file) as conn_loops:
       # list of radars in the file
       avail_radar_list  = [ stn[0] for stn in conn_loops.execute("select distinct ID_STN from header order by 1;").fetchall() ]
-
+      print (avail_radar_list)
       for this_radar in avail_radar_list:
         #select radars if desired radar_list is specified
+        print (this_radar,desired_radar_list != 'all')
         if desired_radar_list != 'all':
-          if this_radar not in desired_radar_list:
+          if this_radar not in avail_radar_list:
             continue
         # dates
         result = conn_loops.execute(f"select distinct DATE, printf('%06d', TIME) from header where ID_STN = '{this_radar}' ;").fetchall()
         datetime_list  = [datetime.datetime.strptime(f'{date_time[0]}{date_time[1]}', '%Y%m%d%H%M%S') for date_time in result]
         datetime_list.sort()
-
+        print ("DAVID",datetime_list)
         for this_date in datetime_list:
           #only include date in assimilation window
+          print (this_date ,'>', window_t0, this_date ,'<',window_tf)
           if (this_date > window_t0) and (this_date <= window_tf):
             vol_scan_list.append( {'file':this_file, 'radar':this_radar, 'date':this_date} )
-
+  
   if len(vol_scan_list) == 0:
     raise RuntimeError('List of volume scan to process is empty. Check that files are available and search criteria are correct.')
 
@@ -775,16 +795,107 @@ def superobs_sqlite():
    
     sample_file  = vol_scan_list[0]['file']
     # create info and resume_tables in fileout
-    create_info_resume_tables(sample_file, conn_pathfileout, n_rays, delta_range, ops_run_name)   
-    # test id_obs 
-    distinct_id_obs(conn_pathfileout)
+    create_info_midas_tables(sample_file, conn_pathfileout, n_rays, delta_range, ops_run_name)   
+    # improve searches with index tables
+    create_index_tables_idobs_iddata(conn_pathfileout)
+    # collision_test
+    collision_test(conn_pathfileout)
     # close connection
     conn_pathfileout.close()
     tc_1 = time.time()
    
   print(f"Runtime total: {round(tc_1-tc_0,4)} s")
 
-if __name__ == '__main__':
-  #sys.exit is used to the return status of main is catched and passed to caller
-  sys.exit( superobs_sqlite() )
+def arg_call():
+    import argparse 
 
+    parser = argparse.ArgumentParser()
+    parser.add_argument('--center_time',  default='undefined', type=str,   help="YYYYMMDDHH center time of assimilation window")
+    parser.add_argument('--window_width', default=0.,          type=float, help="[hours] width of assimilation window")
+    parser.add_argument('--desired_radar_list', nargs="+", default='all',          help="List of radars to process")
+    parser.add_argument('--inputfiles', nargs="+", default='undefined',    help="input sqlite file to average")
+    parser.add_argument('--pathin',       default='undefined', type=str,   help="directory where are input sqlite files")
+    parser.add_argument('--infile_struc', default='*/%Y%m%d%H_ra',type=str,help="structure of input file name")
+    parser.add_argument('--pathout',      default=os.getcwd(), type=str,   help="where averaged sqlite files will be put")
+    parser.add_argument('--outfile_struc',default='%Y%m%d%H_superobbed_dvel_ground_radars.sqlite', type=str,   help="structure of output file name")
+    parser.add_argument('--timing',       action="store_true",             help="Average only one PPI for time analysis" ) 
+    parser.add_argument('--ops_run_name', default='N1',        type=str,   help="Name of operational run (will show up in rdb4_schema)" ) 
+    parser.add_argument('--n_rays',       default=20,          type=int,   help="Number of rays for averaging" )
+    parser.add_argument('--delta_range',  default=10000.,      type=float, help="Delta_range  for averaging" )
+    parser.add_argument('--n_cpus',       default=1,           type=int,   help="Number of rays for averaging" )
+
+    args = parser.parse_args()
+
+    if args.inputfiles == 'undefined' and args.pathin == 'undefined':
+
+      args.center_time = '2019073100'
+      args.window_width = 6.
+      args.desired_radar_list = ['casra', 'usdyx']
+      #option1: explicitely specify inputfiles
+      #args.inputfiles = ['/space/hall4/sitestore/eccc/cmd/a/dlo001/data/doppler_qc/doppler_qc_v0.3/sqlite_v1.0.0_qc/split_6h/USVNX/2019073100_ra',
+      #                   '/space/hall4/sitestore/eccc/cmd/a/dlo001/data/doppler_qc/doppler_qc_v0.3/sqlite_v1.0.0_qc/split_6h/CASRA/2019073100_ra']
+      #option2: specify pathin + infile_struc and let Python search for files
+      args.pathin = '/space/hall4/sitestore/eccc/cmd/a/dlo001/data/doppler_qc/doppler_qc_v0.3/sqlite_v1.0.0_qc/split_6h/'
+      args.pathout = './work'
+      args.timing = False
+      args.plot   = False
+      args.n_cpus = 40
+
+      print(f'superobs called with no input filename(s)')
+      print(f'We are running demo with:')
+      for arg in vars(args):
+        print(f'--{arg}  {getattr(args, arg)}')
+
+
+    #argument checking
+    if args.center_time == 'undefined':
+        raise ValueError('Center time of assimilation window must be provided')
+    else:
+       args.center_time = datetime.datetime.strptime(args.center_time, '%Y%m%d%H')
+
+    if np.isclose(args.window_width, 0.) :
+        raise ValueError('Window width must be provided')
+
+    if args.inputfiles != 'undefined':
+      #if inputfiles argument is provided, we use that
+      infile_list = args.inputfiles 
+
+    elif args.pathin != 'undefined': 
+      #alternatively, search for files with pathin+infile_struc 
+      if not os.path.isdir(args.pathin):
+        raise ValueError(f'pathin: {args.pathin} does not exist.')
+      search_str = args.center_time.strftime(args.infile_struc)
+      infile_list = glob.glob(f'{args.pathin}/{search_str}')
+
+    else:
+      raise ValueError('At least one of inputfiles ot pathin must be provided')
+
+    #check infile_list
+    if len(infile_list) == 0:
+      raise ValueError('infile_list is empty, we stop here')
+      sys.exit(1)
+
+    else:
+      for this_file in infile_list:
+        if not os.path.isfile(this_file):
+          raise ValueError(f'inputfiles: {this_file} does not exist.')
+
+    if not os.path.isdir(args.pathout):
+      os.mkdir(args.pathout)
+
+    #sys.exit is used to the return status of main is catched and passed to caller
+    print (infile_list)
+    sys.exit(compute_average(args.center_time,
+                  args.window_width,
+                  args.desired_radar_list,
+                  infile_list, 
+                  args.pathout, 
+                  args.outfile_struc, 
+                  args.timing, 
+                  args.ops_run_name, 
+                  args.n_rays, 
+                  args.delta_range, 
+                  args.n_cpus) )
+
+if __name__ == '__main__':
+    arg_call()
